﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using RimWorld;
using UnityEngine;
using Verse;

namespace GainSkillExpExt
{
    public class ExtendedDoerGainSkillExp : BookOutcomeDoerGainSkillExp
    {
        public override void OnBookGenerated(Pawn author = null)
        {
            BookOutcomeProperties_GainSkillExpExt propsExt = Props as BookOutcomeProperties_GainSkillExpExt;
            if (propsExt == null)
            {
                Log.Error("ExtendedDoerGainSkillExp: Props is not of type BookOutcomeProperties_GainSkillExpExt");
                return;
            }

            FieldInfo valuesField = typeof(BookOutcomeDoerGainSkillExp).GetField("values", BindingFlags.Instance | BindingFlags.NonPublic);
            if (valuesField == null)
            {
                Log.Error("ExtendedDoerGainSkillExp: Unable to find 'values' field via reflection");
                return;
            }

            Dictionary<SkillDef, float> values = valuesField.GetValue(this) as Dictionary<SkillDef, float>;
            if (values == null)
            {
                Log.Error("ExtendedDoerGainSkillExp: 'values' is null");
                return;
            }

            if (propsExt.skills.Count > 0)
            {
                float baseExp = BookUtility.GetSkillExpForQuality(Quality);
                if (propsExt.skills.Count > 1)
                {
                    baseExp *= 1.25f;
                }
                foreach (var reward in propsExt.skills)
                {
                    values[reward.skill] = baseExp;
                }
            }
            else
            {
                base.OnBookGenerated(author);
            }

            // 個別乗数を反映
            foreach (SkillDef skill in values.Keys.ToList())
            {
                float baseValue = values[skill];
                float multiplier = propsExt.GetMultiplierForSkill(skill);
                values[skill] = baseValue * multiplier;
            }
        }

        public override void OnReadingTick(Pawn reader, float factor)
        {
            FieldInfo valuesField = typeof(BookOutcomeDoerGainSkillExp).GetField("values", BindingFlags.Instance | BindingFlags.NonPublic);
            if (valuesField == null)
            {
                Log.Error("ExtendedDoerGainSkillExp: Unable to find 'values' field via reflection");
                return;
            }

            Dictionary<SkillDef, float> values = valuesField.GetValue(this) as Dictionary<SkillDef, float>;
            if (values == null)
            {
                Log.Error("ExtendedDoerGainSkillExp: 'values' is null");
                return;
            }

            foreach (var kvp in values)
            {
                SkillDef skillDef = kvp.Key;
                float xpGain = kvp.Value * factor;
                SkillRecord record = reader.skills.GetSkill(skillDef);
                record.Learn(xpGain);

                while (record.xpSinceLastLevel < 0f && record.Level > GetMinLevelForQuality(Quality))
                {
                    DropSkillLevel(record);
                }
                if (record.xpSinceLastLevel < 0f && record.Level <= GetMinLevelForQuality(Quality))
                {
                    record.xpSinceLastLevel = 0f;
                }
            }
        }

        public override string GetBenefitsString(Pawn reader = null)
        {
            BookOutcomeProperties_GainSkillExpExt propsExt = Props as BookOutcomeProperties_GainSkillExpExt;
            if (propsExt == null)
                return base.GetBenefitsString(reader);

            float baseExp = BookUtility.GetSkillExpForQuality(Quality);
            if (propsExt.skills.Count > 1)
                baseExp *= 1.25f;

            System.Text.StringBuilder sb = new System.Text.StringBuilder();
            foreach (var reward in propsExt.skills)
            {
                float multiplier = propsExt.GetMultiplierForSkill(reward.skill);
                int effectiveCap;
                if (multiplier >= 0)
                {
                    // 正の乗数：元の上限に positiveCapOffset を加算、結果は 1～20 にクランプ
                    effectiveCap = GetBaseMaxSkillLevel(Quality) + (int)propsExt.positiveCapOffset;
                    effectiveCap = Mathf.Clamp(effectiveCap, 1, 20);
                }
                else
                {
                    // 負の乗数：元の下限に negativeCapOffset を加算、結果は 0 以上にクランプ
                    effectiveCap = GetMinLevelForQuality(Quality) + (int)propsExt.negativeCapOffset;
                    effectiveCap = Mathf.Max(effectiveCap, 0);
                }
                float effectiveExp = baseExp * multiplier;
                float xpPerHour = effectiveExp * 2500f;
                sb.AppendLine($" - {reward.skill.label}: {xpPerHour:F0} XP/Hour (Cap: {effectiveCap})");
            }
            return sb.ToString().TrimEnd();
        }

        private int GetBaseMaxSkillLevel(QualityCategory quality)
        {
            switch (quality)
            {
                case QualityCategory.Awful:
                    return 2;
                case QualityCategory.Poor:
                    return 5;
                case QualityCategory.Normal:
                    return 8;
                case QualityCategory.Good:
                    return 10;
                case QualityCategory.Excellent:
                    return 12;
                case QualityCategory.Masterwork:
                    return 14;
                case QualityCategory.Legendary:
                    return 16;
                default:
                    return 0;
            }
        }

        private int GetMinLevelForQuality(QualityCategory quality)
        {
            switch (quality)
            {
                case QualityCategory.Legendary:
                    return 16;
                case QualityCategory.Masterwork:
                    return 12;
                case QualityCategory.Excellent:
                    return 10;
                case QualityCategory.Good:
                    return 8;
                case QualityCategory.Normal:
                    return 6;
                case QualityCategory.Poor:
                    return 4;
                case QualityCategory.Awful:
                    return 0;
                default:
                    return 0;
            }
        }

        private void DropSkillLevel(SkillRecord record)
        {
            Log.Message($"Skill {record.def.defName} dropped from level {record.Level} due to negative XP.");
            record.Level = record.Level - 1;
            float xpRequired = SkillRecord.XpRequiredToLevelUpFrom(record.Level);
            record.xpSinceLastLevel = xpRequired - 1f;
            Log.Message($"Skill {record.def.defName} is now at level {record.Level} with xp set to {record.xpSinceLastLevel}.");
        }
    }
}
